@@ -385,15 +385,15 @@ public abstract class BaseNativePOSIX extends NativePOSIX implements POSIX {
     }
     
     public String readlink(String oldpath) throws IOException {
-        // TODO: this should not be hardcoded to 256 bytes
-        ByteBuffer buffer = ByteBuffer.allocate(256);
+        // TODO: this should not be hardcoded to 1024 bytes
+        ByteBuffer buffer = ByteBuffer.allocate(1024);
         int result = libc().readlink(oldpath, buffer, buffer.capacity());
         
         if (result == -1) return null;
         
         buffer.position(0);
         buffer.limit(result);
-        return Charset.forName("ASCII").decode(buffer).toString();
+        return Charset.defaultCharset().decode(buffer).toString();
     }
 
     public int readlink(CharSequence path, byte[] buf, int bufsize) {
@@ -450,6 +450,34 @@ public abstract class BaseNativePOSIX extends NativePOSIX implements POSIX {
         return libc().lutimes(path, times);
     }
 
+    public int utimensat(int dirfd, String path, long[] atimespec, long[] mtimespec, int flag) {
+        Timespec[] times = null;
+        if (atimespec != null && mtimespec != null) {
+            times = Struct.arrayOf(getRuntime(), DefaultNativeTimespec.class, 2);
+            times[0].setTime(atimespec);
+            times[1].setTime(mtimespec);
+        }
+        return libc().utimensat(dirfd, path, times, flag);
+    }
+
+    public int utimensat(int dirfd, String path, Pointer times, int flag) {
+        return libc().utimensat(dirfd, path, times, flag);
+    }
+
+    public int futimens(int fd, long[] atimespec, long[] mtimespec) {
+        Timespec[] times = null;
+        if (atimespec != null && mtimespec != null) {
+            times = Struct.arrayOf(getRuntime(), DefaultNativeTimespec.class, 2);
+            times[0].setTime(atimespec);
+            times[1].setTime(mtimespec);
+        }
+        return libc().futimens(fd, times);
+    }
+
+    public int futimens(int fd, Pointer times) {
+        return libc().futimens(fd, times);
+    }
+
     public int fork() {
         return libc().fork();
     }
