@@ -98,7 +98,7 @@ public class FileTest {
             // The nano secs part is available in other stat implementations.  We use Linux x86_64 because it's
             // representative.  We really just want to verify that the usec portion of the timeval is passed throug
             // to the POSIX call.
-            if (stat instanceof NanosecondFileStat && !Platform.IS_MAC) {
+            if (hasNanosecondPrecision(stat)) {
                 NanosecondFileStat linuxStat = (NanosecondFileStat) stat;
 
 //                assertEquals("atime useconds failed", 200000, linuxStat.aTimeNanoSecs());
@@ -109,6 +109,41 @@ public class FileTest {
         }
     }
 
+    @Test
+    public void utimensatAbsolutePath() throws Throwable {
+        File file = File.createTempFile("utimensat", null);
+        utimensat(file, 0);
+    }
+
+    @Test
+    public void utimensatRelativePath() throws Throwable {
+        String path = "utimensat";
+        File file = new File(path);
+        file.createNewFile();
+        int parentFd = posix.open(".", OpenFlags.O_DIRECTORY.intValue(), 0444);
+        utimensat(file, parentFd);
+    }
+
+    @Test
+    public void futimens() throws Throwable {
+        File file = File.createTempFile("futimens", null);
+        FileStat fileStat = posix.stat(file.getPath());
+        if (!hasNanosecondPrecision(fileStat)) {
+            file.delete();
+            return;
+        }
+
+        long atimeSeconds = fileStat.atime()+1;
+        long mtimeSeconds = fileStat.mtime()-1;
+        long atimeNanoSeconds = 123456789;
+        long mtimeNanoSeconds = 135;
+        int fd = posix.open(file.getAbsolutePath(), OpenFlags.O_RDWR.intValue(), 0444);
+        posix.futimens(fd,
+                new long[] {atimeSeconds, atimeNanoSeconds},
+                new long[] {mtimeSeconds, mtimeNanoSeconds});
+        assertStatNanoSecond(file, atimeSeconds, atimeNanoSeconds, mtimeSeconds, mtimeNanoSeconds);
+    }
+
     @Test
     public void lutimesTest() throws Throwable {
         // FIXME: On Windows this is working but providing wrong numbers and therefore getting wrong results.
@@ -482,15 +517,19 @@ public class FileTest {
     @Test
     public void readlinkTest() throws IOException {
         if (!Platform.IS_WINDOWS) {
-            File file = File.createTempFile("jnr-posix-readlink-test", "tmp");
+            File file = File.createTempFile("jnr-pøsix-réådlînk-tést", "tmp");
             File link = new File(file.getAbsolutePath() + "link");
 
             posix.symlink(file.getAbsolutePath(), link.getAbsolutePath());
 
-            byte[] buffer = new byte[file.getAbsolutePath().length()];
-            assertEquals(buffer.length, posix.readlink(link.getAbsolutePath(), buffer, buffer.length));
+            byte[] fileBytes = file.getAbsolutePath().getBytes();
+            int fileLength = fileBytes.length;
 
-            assertArrayEquals(buffer, file.getAbsolutePath().getBytes());
+            byte[] buffer = new byte[fileLength];
+
+            assertEquals(fileLength, posix.readlink(link.getAbsolutePath(), buffer, buffer.length));
+
+            assertArrayEquals(fileBytes, buffer);
 
             link.delete();
             file.delete();
@@ -500,13 +539,16 @@ public class FileTest {
     @Test
     public void readlinkByteBufferTest() throws IOException {
         if (!Platform.IS_WINDOWS) {
-            File file = File.createTempFile("jnr-posix-readlink-test", "tmp");
+            File file = File.createTempFile("jnr-pøsix-réådlînk-tést", "tmp");
             File link = new File(file.getAbsolutePath() + "link");
 
             posix.symlink(file.getAbsolutePath(), link.getAbsolutePath());
 
-            ByteBuffer buffer = ByteBuffer.allocate(file.getAbsolutePath().length());
-            assertEquals(buffer.capacity(), posix.readlink(link.getAbsolutePath(), buffer, buffer.capacity()));
+            byte[] fileBytes = file.getAbsolutePath().getBytes();
+            int fileLength = fileBytes.length;
+            ByteBuffer buffer = ByteBuffer.allocate(fileLength);
+
+            assertEquals(fileLength, posix.readlink(link.getAbsolutePath(), buffer, buffer.capacity()));
 
             assertArrayEquals(buffer.array(), file.getAbsolutePath().getBytes());
 
@@ -518,18 +560,21 @@ public class FileTest {
     @Test
     public void readlinkPointerTest() throws IOException {
         if (!Platform.IS_WINDOWS) {
-            File file = File.createTempFile("jnr-posix-readlink-test", "tmp");
+            File file = File.createTempFile("jnr-pøsix-réådlînk-tést", "tmp");
             File link = new File(file.getAbsolutePath() + "link");
 
             int bufSize = 1024;
-            int filenameLength = file.getAbsolutePath().length();
 
             Pointer buffer = jnr.ffi.Runtime.getSystemRuntime().getMemoryManager().allocateDirect(bufSize);
+
             posix.symlink(file.getAbsolutePath(), link.getAbsolutePath());
+            
+            byte[] fileBytes = file.getAbsolutePath().getBytes();
+            int fileLength = fileBytes.length;
 
-            assertEquals(filenameLength, posix.readlink(link.getAbsolutePath(), buffer, bufSize));
+            assertEquals(fileLength, posix.readlink(link.getAbsolutePath(), buffer, bufSize));
 
-            assertEquals(buffer.getString(0, filenameLength, Charset.defaultCharset()), file.getAbsolutePath());
+            assertEquals(file.getAbsolutePath(), buffer.getString(0, fileLength, Charset.defaultCharset()));
 
             link.delete();
             file.delete();
@@ -586,4 +631,37 @@ public class FileTest {
             return JavaLibCHelper.toFileDescriptor(fd);
         }
     }
+
+    private boolean hasNanosecondPrecision(FileStat fileStat) {
+        return fileStat instanceof NanosecondFileStat && !Platform.IS_MAC;
+    }
+
+    private void utimensat(File file, int parentFd) {
+        String path = file.getAbsolutePath();
+        FileStat fileStat = posix.stat(path);
+        if (!hasNanosecondPrecision(fileStat)) {
+            file.delete();
+            return;
+        }
+
+        long atimeSeconds = fileStat.atime()+2;
+        long mtimeSeconds = fileStat.mtime()-2;
+        long atimeNanoSeconds = 123456789;
+        long mtimeNanoSeconds = 135;
+        posix.utimensat(parentFd,
+                path,
+                new long[] {atimeSeconds, atimeNanoSeconds},
+                new long[] {mtimeSeconds, mtimeNanoSeconds},
+                0);
+        assertStatNanoSecond(file, atimeSeconds, atimeNanoSeconds, mtimeSeconds, mtimeNanoSeconds);
+    }
+
+    private void assertStatNanoSecond(File file, long atimeSeconds, long atimeNanoSeconds, long mtimeSeconds, long mtimeNanoSeconds) {
+        NanosecondFileStat nanosecondFileStat = (NanosecondFileStat) posix.stat(file.getPath());
+        assertEquals("Access timestamp should be updated", atimeSeconds, nanosecondFileStat.atime());
+        assertEquals("Modification timestamp should be updated", mtimeSeconds, nanosecondFileStat.mtime());
+        assertEquals("Access time precision should be in ns", atimeNanoSeconds, nanosecondFileStat.aTimeNanoSecs());
+        assertEquals("Modification time precision should be in ns", mtimeNanoSeconds, nanosecondFileStat.mTimeNanoSecs());
+        file.delete();
+    }
 }
